// Copyright 2010 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "tictactoe.h"

TicTacToe::TicTacToe(QObject *parent) : QObject(parent) {
	connect(this, SIGNAL(playerWon(int)), this, SLOT(stopGame()));
	reset();
}

void TicTacToe::reset(int beginner) {
	for (int x = 0; x < 3; x++) {
		for (int y = 0; y < 3; y++) {
			state[x][y] = 0;
		}
	}

	currentPlayer = beginner;
	gameOver = false;
	moveCount = 0;
}

int TicTacToe::getState(int x, int y) {
	return state[x][y];
}

int TicTacToe::getNumMoves() {
	return moveCount;
}

void TicTacToe::move(int x, int y, int player) {
	if (currentPlayer != player) return;
	if (gameOver) return;
	if (state[x][y] != 0) return;

	state[x][y] = player;
	moveCount++;
	currentPlayer = 3-currentPlayer;

	checkForVictory();
}

void TicTacToe::stopGame() {
	gameOver = true;
}

void TicTacToe::checkForVictory() {
	//8 ways to win
	//1-6: First/second/third vertical/horizontal line:
	for (int l = 0; l < 3; l++) {
		//Vertical line
		if ((state[l][0] != 0) && (state[l][0] == state[l][1]) && (state[l][0] == state[l][2])) emit(playerWon(state[l][0]));

		//horizontal line
		if ((state[0][l] != 0) && (state[0][l] == state[1][l]) && (state[0][l] == state[2][l])) emit(playerWon(state[0][l]));
	}
	//7: Main diag
	if ((state[0][0] != 0) && (state[0][0] == state[1][1]) && (state[0][0] == state[2][2])) emit(playerWon(state[0][0]));

	//8: Other diag
	if ((state[2][0] != 0) && (state[2][0] == state[1][1]) && (state[2][0] == state[0][2])) emit(playerWon(state[2][0]));
}
