// Copyright 2010 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "ttt.h"
#include "ui_ttt.h"

#include <QMessageBox>

TTT::TTT(QWidget *parent) : QWidget(parent), ui(new Ui::TTT) {
	ui->setupUi(this);

	ttt = new TicTacToe(this);
	ai = new TTT_AI(this);
	ui->wScreen->setGame(ttt);

	connect(ttt, SIGNAL(playerWon(int)), this, SLOT(showWinner(int)));
	connect(ai, SIGNAL(madeMove(int,int)), this, SLOT(makeAIMove(int,int)));
	connect(ui->wScreen, SIGNAL(madeMove(int,int)), this, SLOT(makeHumanMove(int,int)));
	connect(ui->bNewGame, SIGNAL(clicked()), this, SLOT(newGame()));

	newGame();
}

TTT::~TTT()
{
    delete ui;
	delete ttt;
}

void TTT::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void TTT::newGame() {
	if (ui->cbBeginner->isChecked()) {
		playerAI = 2;
		playerHuman = 1;
	} else {
		playerAI = 1;
		playerHuman = 2;
	}

	ttt->reset();
	ai->reset(!ui->cbBeginner->isChecked());

	update();
}

void TTT::showWinner(int player) {
	if (player == playerHuman) {
		ui->lcdPlayer->display(ui->lcdPlayer->intValue()+1);
		QMessageBox::information(this, "You win", "You have won the game. Good!");
	} else {
		ui->lcdAi->display(ui->lcdAi->intValue()+1);
		QMessageBox::information(this, "You loose", "You have lost the game. Ha ha!");
	}
}

void TTT::makeAIMove(int x, int y) {
	ttt->move(x, y, playerAI);
	update();
}

void TTT::makeHumanMove(int x, int y) {
	ttt->move(x, y, playerHuman);
	if (ttt->getNumMoves() < 9)
		ai->move(x, y);
	else
		update();
}
