// Copyright 2010 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "ttt_ai.h"
#include <QColor>
#include <QDebug>

TTT_AI::TTT_AI(QObject *parent) : QObject(parent) {
	beginnerImage = QImage(":/beginner.png");
	secondImage  = QImage(":/second.png");

	reset();
}

void TTT_AI::reset(bool beginner) {
	if (beginner) workImage = beginnerImage.copy(0,0,beginnerImage.width(), beginnerImage.height());
	else workImage = secondImage.copy(0,0,secondImage.width(), secondImage.height());

	if (beginner) {
		makeMyMove();
	}
}

void TTT_AI::move(int x, int y) {
	int w = workImage.width()/3;
	workImage = workImage.copy(w*x,w*y,w,w);
	makeMyMove();
}

QPoint TTT_AI::getMyMove() {
	QColor c;
	int w = (workImage.width()-1)/2;
	for (int x = 0; x < 3; x++) {
		for (int y = 0; y < 3; y++) {
			c = QColor(workImage.pixel(w*x,w*y));
			if ((c.red() == 255) && (c.blue() == 0)) return QPoint(x,y);
		}
	}
	throw -1;
}

void TTT_AI::makeMyMove() {
	try {
		QPoint move = getMyMove();
		emit(madeMove(move.x(), move.y()));
	} catch (int e) {
		if (e == -1) {
			qDebug() << "Could not find correct move.";
			workImage.save("moveDBG.png");
		}
	}
}
